@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,11 +25,7 @@
 
 package jdk.internal.foreign;
 
-import java.lang.foreign.MemoryLayout;
-import java.lang.foreign.MemorySegment;
-import java.lang.foreign.SegmentAllocator;
-import java.lang.foreign.SegmentScope;
-import java.lang.foreign.ValueLayout;
+import java.lang.foreign.*;
 import java.lang.reflect.Array;
 import java.nio.Buffer;
 import java.nio.ByteBuffer;
@@ -40,6 +36,7 @@ import java.nio.FloatBuffer;
 import java.nio.IntBuffer;
 import java.nio.LongBuffer;
 import java.nio.ShortBuffer;
+import java.nio.charset.Charset;
 import java.util.*;
 import java.util.function.BiFunction;
 import java.util.function.Consumer;
@@ -47,14 +44,17 @@ import java.util.function.Function;
 import java.util.function.IntFunction;
 import java.util.stream.Stream;
 import java.util.stream.StreamSupport;
+
 import jdk.internal.access.JavaNioAccess;
 import jdk.internal.access.SharedSecrets;
 import jdk.internal.access.foreign.UnmapperProxy;
 import jdk.internal.misc.ScopedMemoryAccess;
-import jdk.internal.misc.Unsafe;
+import jdk.internal.reflect.CallerSensitive;
+import jdk.internal.reflect.Reflection;
 import jdk.internal.util.ArraysSupport;
 import jdk.internal.util.Preconditions;
 import jdk.internal.vm.annotation.ForceInline;
+import sun.nio.ch.DirectBuffer;
 
 import static java.lang.foreign.ValueLayout.JAVA_BYTE;
 
@@ -77,16 +77,16 @@ public abstract sealed class AbstractMemorySegmentImpl
 
     final long length;
     final boolean readOnly;
-    final SegmentScope scope;
+    final MemorySessionImpl scope;
 
     @ForceInline
-    AbstractMemorySegmentImpl(long length, boolean readOnly, SegmentScope scope) {
+    AbstractMemorySegmentImpl(long length, boolean readOnly, MemorySessionImpl scope) {
         this.length = length;
         this.readOnly = readOnly;
         this.scope = scope;
     }
 
-    abstract AbstractMemorySegmentImpl dup(long offset, long size, boolean readOnly, SegmentScope scope);
+    abstract AbstractMemorySegmentImpl dup(long offset, long size, boolean readOnly, MemorySessionImpl scope);
 
     abstract ByteBuffer makeByteBuffer();
 
@@ -112,6 +112,56 @@ public abstract sealed class AbstractMemorySegmentImpl
         return asSliceNoCheck(offset, length - offset);
     }
 
+    @Override
+    public MemorySegment asSlice(long offset, long newSize, long byteAlignment) {
+        checkBounds(offset, newSize);
+        Utils.checkAlign(byteAlignment);
+
+        if (!isAlignedForElement(offset, byteAlignment)) {
+            throw new IllegalArgumentException("Target offset incompatible with alignment constraints");
+        }
+        return asSliceNoCheck(offset, newSize);
+    }
+
+    @Override
+    public MemorySegment asSlice(long offset, MemoryLayout layout) {
+        Objects.requireNonNull(layout);
+        return asSlice(offset, layout.byteSize(), layout.byteAlignment());
+    }
+
+    @Override
+    @CallerSensitive
+    public final MemorySegment reinterpret(long newSize, Arena arena, Consumer<MemorySegment> cleanup) {
+        Objects.requireNonNull(arena);
+        return reinterpretInternal(Reflection.getCallerClass(), newSize,
+                MemorySessionImpl.toMemorySession(arena), cleanup);
+    }
+
+    @Override
+    @CallerSensitive
+    public final MemorySegment reinterpret(long newSize) {
+        return reinterpretInternal(Reflection.getCallerClass(), newSize, scope, null);
+    }
+
+    @Override
+    @CallerSensitive
+    public final MemorySegment reinterpret(Arena arena, Consumer<MemorySegment> cleanup) {
+        Objects.requireNonNull(arena);
+        return reinterpretInternal(Reflection.getCallerClass(), byteSize(),
+                MemorySessionImpl.toMemorySession(arena), cleanup);
+    }
+
+    public MemorySegment reinterpretInternal(Class<?> callerClass, long newSize, Scope scope, Consumer<MemorySegment> cleanup) {
+        Reflection.ensureNativeAccess(callerClass, MemorySegment.class, "reinterpret");
+        Utils.checkNonNegativeArgument(newSize, "newSize");
+        if (!isNative()) throw new UnsupportedOperationException("Not a native segment");
+        Runnable action = cleanup != null ?
+                () -> cleanup.accept(SegmentFactories.makeNativeSegmentUnchecked(address(), newSize)) :
+                null;
+        return SegmentFactories.makeNativeSegmentUnchecked(address(), newSize,
+                (MemorySessionImpl)scope, readOnly, action);
+    }
+
     private AbstractMemorySegmentImpl asSliceNoCheck(long offset, long newSize) {
         return dup(offset, newSize, readOnly, scope);
     }
@@ -122,7 +172,7 @@ public abstract sealed class AbstractMemorySegmentImpl
         if (elementLayout.byteSize() == 0) {
             throw new IllegalArgumentException("Element layout size cannot be zero");
         }
-        Utils.checkElementAlignment(elementLayout, "Element layout alignment greater than its size");
+        Utils.checkElementAlignment(elementLayout, "Element layout size is not multiple of alignment");
         if (!isAlignedForElement(0, elementLayout)) {
             throw new IllegalArgumentException("Incompatible alignment constraints");
         }
@@ -148,7 +198,7 @@ public abstract sealed class AbstractMemorySegmentImpl
     @Override
     public MemorySegment allocate(long byteSize, long byteAlignment) {
         Utils.checkAllocationSizeAndAlign(byteSize, byteAlignment);
-        return asSlice(0, byteSize);
+        return asSlice(0, byteSize, byteAlignment);
     }
 
     /**
@@ -219,7 +269,7 @@ public abstract sealed class AbstractMemorySegmentImpl
             final long thatEnd = thatStart + that.byteSize();
 
             if (thisStart < thatEnd && thisEnd > thatStart) {  //overlap occurs
-                long offsetToThat = this.segmentOffset(that);
+                long offsetToThat = that.address() - this.address();
                 long newOffset = offsetToThat >= 0 ? offsetToThat : 0;
                 return Optional.of(asSlice(newOffset, Math.min(this.byteSize() - newOffset, that.byteSize() + offsetToThat)));
             }
@@ -228,12 +278,15 @@ public abstract sealed class AbstractMemorySegmentImpl
     }
 
     @Override
-    public final long segmentOffset(MemorySegment other) {
-        AbstractMemorySegmentImpl that = (AbstractMemorySegmentImpl) Objects.requireNonNull(other);
-        if (unsafeGetBase() == that.unsafeGetBase()) {
-            return that.unsafeGetOffset() - this.unsafeGetOffset();
-        }
-        throw new UnsupportedOperationException("Cannot compute offset from native to heap (or vice versa).");
+    public MemorySegment copyFrom(MemorySegment src) {
+        MemorySegment.copy(src, 0, this, 0, src.byteSize());
+        return this;
+    }
+
+    @Override
+    public long mismatch(MemorySegment other) {
+        Objects.requireNonNull(other);
+        return MemorySegment.mismatch(this, 0, byteSize(), other, 0, other.byteSize());
     }
 
     @Override
@@ -304,10 +357,15 @@ public abstract sealed class AbstractMemorySegmentImpl
     }
 
     @ForceInline
-    public void checkAccess(long offset, long length, boolean readOnly) {
+    public void checkReadOnly(boolean readOnly) {
         if (!readOnly && this.readOnly) {
-            throw new UnsupportedOperationException("Attempt to write a read-only segment");
+            throw new IllegalArgumentException("Attempt to write a read-only segment");
         }
+    }
+
+    @ForceInline
+    public void checkAccess(long offset, long length, boolean readOnly) {
+        checkReadOnly(readOnly);
         checkBounds(offset, length);
     }
 
@@ -325,7 +383,12 @@ public abstract sealed class AbstractMemorySegmentImpl
 
     @ForceInline
     public final boolean isAlignedForElement(long offset, MemoryLayout layout) {
-        return (((unsafeGetOffset() + offset) | maxAlignMask()) & (layout.byteAlignment() - 1)) == 0;
+        return isAlignedForElement(offset, layout.byteAlignment());
+    }
+
+    @ForceInline
+    public final boolean isAlignedForElement(long offset, long byteAlignment) {
+        return (((unsafeGetOffset() + offset) | maxAlignMask()) & (byteAlignment - 1)) == 0;
     }
 
     private int checkArraySize(String typeName, int elemSize) {
@@ -358,13 +421,18 @@ public abstract sealed class AbstractMemorySegmentImpl
     }
 
     @Override
-    public SegmentScope scope() {
+    public Scope scope() {
         return scope;
     }
 
+    @Override
+    public boolean isAccessibleBy(Thread thread) {
+        return sessionImpl().isAccessibleBy(thread);
+    }
+
     @ForceInline
     public final MemorySessionImpl sessionImpl() {
-        return (MemorySessionImpl)scope;
+        return scope;
     }
 
     private IndexOutOfBoundsException outOfBoundException(long offset, long length) {
@@ -450,7 +518,11 @@ public abstract sealed class AbstractMemorySegmentImpl
 
     @Override
     public String toString() {
-        return "MemorySegment{ array: " + array() + " address:" + address() + " limit: " + length + " }";
+        return "MemorySegment{ " +
+                heapBase().map(hb -> "heapBase: " + hb + ", ").orElse("") +
+                "address: " + Utils.toHexString(address()) +
+                ", byteSize: " + length +
+                " }";
     }
 
     @Override
@@ -481,57 +553,42 @@ public abstract sealed class AbstractMemorySegmentImpl
         int size = limit - pos;
 
         AbstractMemorySegmentImpl bufferSegment = (AbstractMemorySegmentImpl) NIO_ACCESS.bufferSegment(bb);
-        final SegmentScope bufferScope;
+        boolean readOnly = bb.isReadOnly();
+        int scaleFactor = getScaleFactor(bb);
+        final MemorySessionImpl bufferScope;
         if (bufferSegment != null) {
             bufferScope = bufferSegment.scope;
         } else {
-            bufferScope = MemorySessionImpl.heapSession(bb);
+            bufferScope = MemorySessionImpl.createHeap(bufferRef(bb));
         }
-        boolean readOnly = bb.isReadOnly();
-        int scaleFactor = getScaleFactor(bb);
+        long off = bbAddress + ((long)pos << scaleFactor);
+        long len = (long)size << scaleFactor;
         if (base != null) {
-            if (base instanceof byte[]) {
-                return new HeapMemorySegmentImpl.OfByte(bbAddress + (pos << scaleFactor), base, size << scaleFactor, readOnly);
-            } else if (base instanceof short[]) {
-                return new HeapMemorySegmentImpl.OfShort(bbAddress + (pos << scaleFactor), base, size << scaleFactor, readOnly);
-            } else if (base instanceof char[]) {
-                return new HeapMemorySegmentImpl.OfChar(bbAddress + (pos << scaleFactor), base, size << scaleFactor, readOnly);
-            } else if (base instanceof int[]) {
-                return new HeapMemorySegmentImpl.OfInt(bbAddress + (pos << scaleFactor), base, size << scaleFactor, readOnly);
-            } else if (base instanceof float[]) {
-                return new HeapMemorySegmentImpl.OfFloat(bbAddress + (pos << scaleFactor), base, size << scaleFactor, readOnly);
-            } else if (base instanceof long[]) {
-                return new HeapMemorySegmentImpl.OfLong(bbAddress + (pos << scaleFactor), base, size << scaleFactor, readOnly);
-            } else if (base instanceof double[]) {
-                return new HeapMemorySegmentImpl.OfDouble(bbAddress + (pos << scaleFactor), base, size << scaleFactor, readOnly);
-            } else {
-                throw new AssertionError("Cannot get here");
-            }
+            return switch (base) {
+                case byte[]   _ -> new HeapMemorySegmentImpl.OfByte(off, base, len, readOnly, bufferScope);
+                case short[]  _ -> new HeapMemorySegmentImpl.OfShort(off, base, len, readOnly, bufferScope);
+                case char[]   _ -> new HeapMemorySegmentImpl.OfChar(off, base, len, readOnly, bufferScope);
+                case int[]    _ -> new HeapMemorySegmentImpl.OfInt(off, base, len, readOnly, bufferScope);
+                case float[]  _ -> new HeapMemorySegmentImpl.OfFloat(off, base, len, readOnly, bufferScope);
+                case long[]   _ -> new HeapMemorySegmentImpl.OfLong(off, base, len, readOnly, bufferScope);
+                case double[] _ -> new HeapMemorySegmentImpl.OfDouble(off, base, len, readOnly, bufferScope);
+                default         -> throw new AssertionError("Cannot get here");
+            };
         } else if (unmapper == null) {
-            return new NativeMemorySegmentImpl(bbAddress + (pos << scaleFactor), size << scaleFactor, readOnly, bufferScope);
+            return new NativeMemorySegmentImpl(off, len, readOnly, bufferScope);
         } else {
-            // we can ignore scale factor here, a mapped buffer is always a byte buffer, so scaleFactor == 0.
-            return new MappedMemorySegmentImpl(bbAddress + pos, unmapper, size, readOnly, bufferScope);
+            return new MappedMemorySegmentImpl(off, unmapper, len, readOnly, bufferScope);
         }
     }
 
-    private static int getScaleFactor(Buffer buffer) {
-        if (buffer instanceof ByteBuffer) {
-            return 0;
-        } else if (buffer instanceof CharBuffer) {
-            return 1;
-        } else if (buffer instanceof ShortBuffer) {
-            return 1;
-        } else if (buffer instanceof IntBuffer) {
-            return 2;
-        } else if (buffer instanceof FloatBuffer) {
-            return 2;
-        } else if (buffer instanceof LongBuffer) {
-            return 3;
-        } else if (buffer instanceof DoubleBuffer) {
-            return 3;
+    private static Object bufferRef(Buffer buffer) {
+        if (buffer instanceof DirectBuffer directBuffer) {
+            // direct buffer, return either the buffer attachment (for slices and views), or the buffer itself
+            return directBuffer.attachment() != null ?
+                    directBuffer.attachment() : directBuffer;
         } else {
-            throw new AssertionError("Cannot get here");
+            // heap buffer, return the underlying array
+            return NIO_ACCESS.getBufferBase(buffer);
         }
     }
 
@@ -540,6 +597,7 @@ public abstract sealed class AbstractMemorySegmentImpl
                             MemorySegment dstSegment, ValueLayout dstElementLayout, long dstOffset,
                             long elementCount) {
 
+        Utils.checkNonNegativeIndex(elementCount, "elementCount");
         AbstractMemorySegmentImpl srcImpl = (AbstractMemorySegmentImpl)srcSegment;
         AbstractMemorySegmentImpl dstImpl = (AbstractMemorySegmentImpl)dstSegment;
         if (srcElementLayout.byteSize() != dstElementLayout.byteSize()) {
@@ -571,28 +629,26 @@ public abstract sealed class AbstractMemorySegmentImpl
     public static void copy(MemorySegment srcSegment, ValueLayout srcLayout, long srcOffset,
                             Object dstArray, int dstIndex,
                             int elementCount) {
-
-        long baseAndScale = getBaseAndScale(dstArray.getClass());
+        Utils.checkNonNegativeIndex(elementCount, "elementCount");
+        var dstInfo = Utils.BaseAndScale.of(dstArray);
         if (dstArray.getClass().componentType() != srcLayout.carrier()) {
             throw new IllegalArgumentException("Incompatible value layout: " + srcLayout);
         }
-        int dstBase = (int)baseAndScale;
-        long dstWidth = (int)(baseAndScale >> 32); // Use long arithmetics below
         AbstractMemorySegmentImpl srcImpl = (AbstractMemorySegmentImpl)srcSegment;
         Utils.checkElementAlignment(srcLayout, "Source layout alignment greater than its size");
         if (!srcImpl.isAlignedForElement(srcOffset, srcLayout)) {
             throw new IllegalArgumentException("Source segment incompatible with alignment constraints");
         }
-        srcImpl.checkAccess(srcOffset, elementCount * dstWidth, true);
+        srcImpl.checkAccess(srcOffset, elementCount * dstInfo.scale(), true);
         Objects.checkFromIndexSize(dstIndex, elementCount, Array.getLength(dstArray));
-        if (dstWidth == 1 || srcLayout.order() == ByteOrder.nativeOrder()) {
+        if (dstInfo.scale() == 1 || srcLayout.order() == ByteOrder.nativeOrder()) {
             ScopedMemoryAccess.getScopedMemoryAccess().copyMemory(srcImpl.sessionImpl(), null,
                     srcImpl.unsafeGetBase(), srcImpl.unsafeGetOffset() + srcOffset,
-                    dstArray, dstBase + (dstIndex * dstWidth), elementCount * dstWidth);
+                    dstArray, dstInfo.base() + (dstIndex * dstInfo.scale()), elementCount * dstInfo.scale());
         } else {
             ScopedMemoryAccess.getScopedMemoryAccess().copySwapMemory(srcImpl.sessionImpl(), null,
                     srcImpl.unsafeGetBase(), srcImpl.unsafeGetOffset() + srcOffset,
-                    dstArray, dstBase + (dstIndex * dstWidth), elementCount * dstWidth, dstWidth);
+                    dstArray, dstInfo.base() + (dstIndex * dstInfo.scale()), elementCount * dstInfo.scale(), dstInfo.scale());
         }
     }
 
@@ -600,28 +656,25 @@ public abstract sealed class AbstractMemorySegmentImpl
     public static void copy(Object srcArray, int srcIndex,
                             MemorySegment dstSegment, ValueLayout dstLayout, long dstOffset,
                             int elementCount) {
-
-        long baseAndScale = getBaseAndScale(srcArray.getClass());
+        var srcInfo = Utils.BaseAndScale.of(srcArray);
         if (srcArray.getClass().componentType() != dstLayout.carrier()) {
             throw new IllegalArgumentException("Incompatible value layout: " + dstLayout);
         }
-        int srcBase = (int)baseAndScale;
-        long srcWidth = (int)(baseAndScale >> 32); // Use long arithmetics below
         Objects.checkFromIndexSize(srcIndex, elementCount, Array.getLength(srcArray));
         AbstractMemorySegmentImpl destImpl = (AbstractMemorySegmentImpl)dstSegment;
         Utils.checkElementAlignment(dstLayout, "Destination layout alignment greater than its size");
         if (!destImpl.isAlignedForElement(dstOffset, dstLayout)) {
             throw new IllegalArgumentException("Destination segment incompatible with alignment constraints");
         }
-        destImpl.checkAccess(dstOffset, elementCount * srcWidth, false);
-        if (srcWidth == 1 || dstLayout.order() == ByteOrder.nativeOrder()) {
+        destImpl.checkAccess(dstOffset, elementCount * srcInfo.scale(), false);
+        if (srcInfo.scale() == 1 || dstLayout.order() == ByteOrder.nativeOrder()) {
             ScopedMemoryAccess.getScopedMemoryAccess().copyMemory(null, destImpl.sessionImpl(),
-                    srcArray, srcBase + (srcIndex * srcWidth),
-                    destImpl.unsafeGetBase(), destImpl.unsafeGetOffset() + dstOffset, elementCount * srcWidth);
+                    srcArray, srcInfo.base() + (srcIndex * srcInfo.scale()),
+                    destImpl.unsafeGetBase(), destImpl.unsafeGetOffset() + dstOffset, elementCount * srcInfo.scale());
         } else {
             ScopedMemoryAccess.getScopedMemoryAccess().copySwapMemory(null, destImpl.sessionImpl(),
-                    srcArray, srcBase + (srcIndex * srcWidth),
-                    destImpl.unsafeGetBase(), destImpl.unsafeGetOffset() + dstOffset, elementCount * srcWidth, srcWidth);
+                    srcArray, srcInfo.base() + (srcIndex * srcInfo.scale()),
+                    destImpl.unsafeGetBase(), destImpl.unsafeGetOffset() + dstOffset, elementCount * srcInfo.scale(), srcInfo.scale());
         }
     }
 
@@ -633,10 +686,6 @@ public abstract sealed class AbstractMemorySegmentImpl
         long dstBytes = dstToOffset - dstFromOffset;
         srcImpl.checkAccess(srcFromOffset, srcBytes, true);
         dstImpl.checkAccess(dstFromOffset, dstBytes, true);
-        if (dstImpl == srcImpl) {
-            srcImpl.checkValidState();
-            return -1;
-        }
 
         long bytes = Math.min(srcBytes, dstBytes);
         long i = 0;
@@ -663,23 +712,274 @@ public abstract sealed class AbstractMemorySegmentImpl
         return srcBytes != dstBytes ? bytes : -1;
     }
 
-    private static long getBaseAndScale(Class<?> arrayType) {
-        if (arrayType.equals(byte[].class)) {
-            return (long) Unsafe.ARRAY_BYTE_BASE_OFFSET | ((long)Unsafe.ARRAY_BYTE_INDEX_SCALE << 32);
-        } else if (arrayType.equals(char[].class)) {
-            return (long) Unsafe.ARRAY_CHAR_BASE_OFFSET | ((long)Unsafe.ARRAY_CHAR_INDEX_SCALE << 32);
-        } else if (arrayType.equals(short[].class)) {
-            return (long)Unsafe.ARRAY_SHORT_BASE_OFFSET | ((long)Unsafe.ARRAY_SHORT_INDEX_SCALE << 32);
-        } else if (arrayType.equals(int[].class)) {
-            return (long)Unsafe.ARRAY_INT_BASE_OFFSET | ((long) Unsafe.ARRAY_INT_INDEX_SCALE << 32);
-        } else if (arrayType.equals(float[].class)) {
-            return (long)Unsafe.ARRAY_FLOAT_BASE_OFFSET | ((long)Unsafe.ARRAY_FLOAT_INDEX_SCALE << 32);
-        } else if (arrayType.equals(long[].class)) {
-            return (long)Unsafe.ARRAY_LONG_BASE_OFFSET | ((long)Unsafe.ARRAY_LONG_INDEX_SCALE << 32);
-        } else if (arrayType.equals(double[].class)) {
-            return (long)Unsafe.ARRAY_DOUBLE_BASE_OFFSET | ((long)Unsafe.ARRAY_DOUBLE_INDEX_SCALE << 32);
-        } else {
-            throw new IllegalArgumentException("Not a supported array class: " + arrayType.getSimpleName());
-        }
+    private static int getScaleFactor(Buffer buffer) {
+        return switch (buffer) {
+            case ByteBuffer   _                 -> 0;
+            case CharBuffer   _, ShortBuffer  _ -> 1;
+            case IntBuffer    _, FloatBuffer  _ -> 2;
+            case LongBuffer   _, DoubleBuffer _ -> 3;
+        };
+    }
+
+    // accessors
+
+    @ForceInline
+    @Override
+    public byte get(ValueLayout.OfByte layout, long offset) {
+        return (byte) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfByte layout, long offset, byte value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public boolean get(ValueLayout.OfBoolean layout, long offset) {
+        return (boolean) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfBoolean layout, long offset, boolean value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public char get(ValueLayout.OfChar layout, long offset) {
+        return (char) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfChar layout, long offset, char value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public short get(ValueLayout.OfShort layout, long offset) {
+        return (short) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfShort layout, long offset, short value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public int get(ValueLayout.OfInt layout, long offset) {
+        return (int) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfInt layout, long offset, int value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public float get(ValueLayout.OfFloat layout, long offset) {
+        return (float) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfFloat layout, long offset, float value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public long get(ValueLayout.OfLong layout, long offset) {
+        return (long) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfLong layout, long offset, long value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public double get(ValueLayout.OfDouble layout, long offset) {
+        return (double) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(ValueLayout.OfDouble layout, long offset, double value) {
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public MemorySegment get(AddressLayout layout, long offset) {
+        return (MemorySegment) layout.varHandle().get((MemorySegment)this, offset);
+    }
+
+    @ForceInline
+    @Override
+    public void set(AddressLayout layout, long offset, MemorySegment value) {
+        Objects.requireNonNull(value);
+        layout.varHandle().set((MemorySegment)this, offset, value);
+    }
+
+    @ForceInline
+    @Override
+    public byte getAtIndex(ValueLayout.OfByte layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (byte) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public boolean getAtIndex(ValueLayout.OfBoolean layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (boolean) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public char getAtIndex(ValueLayout.OfChar layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (char) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfChar layout, long index, char value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public short getAtIndex(ValueLayout.OfShort layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (short) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfByte layout, long index, byte value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfBoolean layout, long index, boolean value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfShort layout, long index, short value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public int getAtIndex(ValueLayout.OfInt layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (int) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfInt layout, long index, int value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public float getAtIndex(ValueLayout.OfFloat layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (float) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfFloat layout, long index, float value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public long getAtIndex(ValueLayout.OfLong layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (long) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfLong layout, long index, long value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public double getAtIndex(ValueLayout.OfDouble layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (double) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(ValueLayout.OfDouble layout, long index, double value) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @ForceInline
+    @Override
+    public MemorySegment getAtIndex(AddressLayout layout, long index) {
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        return (MemorySegment) layout.varHandle().get((MemorySegment)this, index * layout.byteSize());
+    }
+
+    @ForceInline
+    @Override
+    public void setAtIndex(AddressLayout layout, long index, MemorySegment value) {
+        Objects.requireNonNull(value);
+        Utils.checkElementAlignment(layout, "Layout alignment greater than its size");
+        layout.varHandle().set((MemorySegment)this, index * layout.byteSize(), value);
+    }
+
+    @Override
+    public String getString(long offset) {
+        return getString(offset, sun.nio.cs.UTF_8.INSTANCE);
+    }
+
+    @Override
+    public String getString(long offset, Charset charset) {
+        Objects.requireNonNull(charset);
+        return StringSupport.read(this, offset, charset);
+    }
+
+    @Override
+    public void setString(long offset, String str) {
+        Objects.requireNonNull(str);
+        setString(offset, str, sun.nio.cs.UTF_8.INSTANCE);
+    }
+
+    @Override
+    public void setString(long offset, String str, Charset charset) {
+        Objects.requireNonNull(charset);
+        Objects.requireNonNull(str);
+        StringSupport.write(this, offset, charset, str);
     }
 }
