@@ -22,6 +22,7 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.ClassUse_FieldAnnotations", "Felder in {1} mit Annotationen vom Typ {0}" },
             { "doclet.ClassUse_FieldTypeParameter", "Felder in {1} mit Typparametern vom Typ {0}" },
             { "doclet.ClassUse_ImplementingClass", "Klassen in {1}, die {0} implementieren" },
+            { "doclet.ClassUse_ImplementsTypeParameter", "Klassen in {1}, die Schnittstellen mit Typargumenten vom Typ {0} implementieren" },
             { "doclet.ClassUse_MethodAnnotations", "Methoden in {1} mit Annotationen vom Typ {0}" },
             { "doclet.ClassUse_MethodArgs", "Methoden in {1} mit Parametern vom Typ {0}" },
             { "doclet.ClassUse_MethodArgsTypeParameters", "Methodenparameter in {1} mit Typargumenten vom Typ {0}" },
@@ -34,7 +35,9 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.ClassUse_PackageAnnotation", "Packages mit Annotationen vom Typ {0}" },
             { "doclet.ClassUse_Packages.that.use.0", "Packages, die {0} verwenden" },
             { "doclet.ClassUse_Subclass", "Unterklassen von {0} in {1}" },
+            { "doclet.ClassUse_SubclassTypeParameter", "Unterklassen mit Typargumenten vom Typ {0} in {1}" },
             { "doclet.ClassUse_Subinterface", "Unterschnittstellen von {0} in {1}" },
+            { "doclet.ClassUse_SubinterfaceTypeParameter", "Unterschnittstellen mit Typargumenten vom Typ {0} in {1}" },
             { "doclet.ClassUse_Title", "Verwendungen von {0}" },
             { "doclet.ClassUse_TypeParameter", "Klassen in {1} mit Typparametern vom Typ {0}" },
             { "doclet.ClassUse_Uses.of.0.in.1", "Verwendungen von {0} in {1}" },
@@ -45,6 +48,7 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.Declared_Using_Preview.SEALED", "Verschl\u00FCsselte Klassen" },
             { "doclet.Declared_Using_Preview.SEALED_PERMITS", "Verschl\u00FCsselte Klassen" },
             { "doclet.Deprecated_API", "Veraltete API" },
+            { "doclet.Deprecated_API_Checkbox_All_Releases", "Alle" },
             { "doclet.Deprecated_API_Checkbox_Label", "Veraltete API anzeigen in:" },
             { "doclet.Deprecated_API_Checkbox_Other_Releases", "Sonstige" },
             { "doclet.Deprecated_Elements", "Veraltete {0}" },
@@ -98,6 +102,7 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.Module_Sub_Nav", "Modul:" },
             { "doclet.Navigation", "Navigation" },
             { "doclet.New_API", "Neue API" },
+            { "doclet.New_API_Checkbox_All_Releases", "Alle" },
             { "doclet.New_API_Checkbox_Label", "Hinzugef\u00FCgte API anzeigen in:" },
             { "doclet.New_Elements", "Neue {0}" },
             { "doclet.New_Elements_Release_Column_Header", "Hinzugef\u00FCgt in" },
@@ -122,6 +127,7 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.PreviewTrailingNote2", "Vorschaufeatures k\u00F6nnen in k\u00FCnftigen Releases entfernt oder zu permanenten Features der Java-Plattform hochgestuft werden." },
             { "doclet.Preview_API", "Vorschau-API" },
             { "doclet.Preview_API_Checkbox_Label", "Vorschau-API anzeigen f\u00FCr:" },
+            { "doclet.Preview_API_Checkbox_Toggle_All", "Alle umschalten" },
             { "doclet.Preview_JEP_URL", "https://openjdk.org/jeps/{0}" },
             { "doclet.Preview_Label", "Vorschau" },
             { "doclet.Preview_Mark", "PREVIEW" },
@@ -131,8 +137,14 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.ReflectivePreviewAPI", "{0} bezieht sich auf mindestens eine reflektive Vorschau-API: {1}." },
             { "doclet.ReflectivePreviewPlatformLeadingNote", "{0} ist eine reflektive Vorschau-API der Java-Plattform." },
             { "doclet.Resource_error", "Fehler beim Lesen der Ressource: {0}" },
+            { "doclet.RestrictedLeadingNote", "{0} ist eine eingeschr\u00E4nkte Methode der Java-Plattform." },
+            { "doclet.RestrictedTrailingNote1", "Programme k\u00F6nnen {0} nur verwenden, wenn der Zugriff auf eingeschr\u00E4nkte Methoden aktiviert ist." },
+            { "doclet.RestrictedTrailingNote2", "Eingeschr\u00E4nkte Methoden sind nicht sicher und k\u00F6nnen bei falscher Verwendung die JVM zum Absturz bringen oder zu einer Besch\u00E4digung des Arbeitsspeichers f\u00FChren." },
+            { "doclet.Restricted_Mark", "RESTRICTED" },
+            { "doclet.Restricted_Methods", "Eingeschr\u00E4nkte Methoden" },
             { "doclet.Same_element_name_used", "Elementname oder Muster zweimal verwendet: {0}" },
             { "doclet.Search_tag_in", "Tag suchen in {0}" },
+            { "doclet.Section", "Abschnitt" },
             { "doclet.Skip_navigation_links", "Navigations-Links \u00FCberspringen" },
             { "doclet.Specification", "Spezifikation" },
             { "doclet.Specified_By", "Angegeben von:" },
@@ -158,6 +170,7 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.Window_New_List", "Neue API-Liste" },
             { "doclet.Window_Overview_Summary", "\u00DCberblick" },
             { "doclet.Window_Preview_List", "Vorschauliste" },
+            { "doclet.Window_Restricted_List", "Liste eingeschr\u00E4nkter Methoden" },
             { "doclet.Window_Search_title", "Suchen" },
             { "doclet.Window_Single_Index", "Index" },
             { "doclet.Window_Source_title", "Quellcode" },
@@ -219,6 +232,7 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.help.page_kinds.head", "Seitenarten" },
             { "doclet.help.page_kinds.intro", "Die folgenden Abschnitte beschreiben die verschiedenen Seitenarten in dieser Collection." },
             { "doclet.help.preview.body", "Die Seite {0} listet alle Vorschau-APIs auf. Vorschau-APIs k\u00F6nnen in k\u00FCnftigen Implementierungen entfernt werden." },
+            { "doclet.help.restricted.body", "Auf der Seite {0} sind alle eingeschr\u00E4nkten Methoden aufgelistet. Eingeschr\u00E4nkte Methoden sind nicht sicher und k\u00F6nnen bei falscher Verwendung die JVM zum Absturz bringen oder zu einer Besch\u00E4digung des Arbeitsspeichers f\u00FChren." },
             { "doclet.help.search.example", "{0} stimmt mit {1} \u00FCberein" },
             { "doclet.help.search.head", "Suchen" },
             { "doclet.help.search.intro", "Sie k\u00F6nnen nach Definitionen von Modulen, Packages, Typen, Feldern, Methoden, Systemeigenschaften und anderen Begriffen suchen, die in der API definiert sind. Dazu k\u00F6nnen Sie den Namen ganz oder teilweise oder optional auch Abk\u00FCrzungen mit Binnenmajuskeln (\"camelCase\") eingeben. Sie k\u00F6nnen auch mehrere durch Leerzeichen getrennte Suchbegriffe angeben. Beispiele:" },
@@ -231,12 +245,15 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.help.tree.intro", "Es gibt eine Seite {0} f\u00FCr alle Packages, und f\u00FCr jedes Package gibt es eine Hierarchie. Jede Hierarchieseite enth\u00E4lt eine Klassen- und eine Schnittstellenliste. Die Klassen sind nach Vererbungsstruktur organisiert, beginnend mit {1}. Die Schnittstellen erben nicht von {1}." },
             { "doclet.help.tree.overview", "Wenn Sie auf der \u00DCbersichtsseite auf \"Baum\" klicken, wird die Hierarchie f\u00FCr alle Packages angezeigt." },
             { "doclet.help.tree.package", "Wenn Sie eine bestimmte Package-, Klassen- oder Schnittstellenseite anzeigen und auf \"Baum\" klicken, wird die Hierarchie nur f\u00FCr dieses Package angezeigt." },
-            { "doclet.help.use.body", "F\u00FCr jedes dokumentierte Package sowie jede Klasse und jede Schnittstelle ist eine eigene Verwendungsseite vorhanden. Auf dieser Seite wird beschrieben, welche Packages, Klassen, Methoden, Konstruktoren und Felder einen Teil der angegebenen Klasse oder des angegebenen Packages verwenden. Bei der Klasse oder Schnittstelle A enth\u00E4lt die Verwendungsseite die Unterklassen von A, als A deklarierte Felder, Methoden, die A zur\u00FCckgeben, sowie Methoden und Konstruktoren mit Parametern des Typs A. Sie k\u00F6nnen diese Seite aufrufen, indem Sie zun\u00E4chst das Package, die Klasse oder die Schnittstelle aufrufen und anschlie\u00DFend in der Navigationsleiste auf den Link \"Verwendung\" klicken." },
+            { "doclet.help.use.body", "F\u00FCr jedes dokumentierte Package sowie jede Klasse oder Schnittstelle ist eine eigene Verwendungsseite vorhanden. Auf dieser Seite werden die Packages, Klassen, Schnittstellen, Methoden, Konstruktoren und Felder aufgelistet, die einen Teil der angegebenen Klasse oder Schnittstelle oder des angegebenen Packages verwenden. Bei der Klasse oder Schnittstelle A enth\u00E4lt die Verwendungsseite die Unterklassen oder Unterschnittstellen von A, als A deklarierte Felder, Methoden, die A zur\u00FCckgeben, Methoden und Konstruktoren mit Parametern des Typs A und Unterklassen oder Unterschnittstellen mit Parametern des Typs A. Sie k\u00F6nnen auf diese Seite zugreifen, indem Sie zun\u00E4chst das Package, die Klasse oder die Schnittstelle aufrufen und anschlie\u00DFend in der Navigationsleiste auf den Link \"Verwendung\" klicken." },
             { "doclet.help.use.head", "Verwendung" },
             { "doclet.in_class", "in Klasse" },
             { "doclet.in_interface", "in Schnittstelle" },
             { "doclet.link.no_reference", "Keine Referenz angegeben" },
             { "doclet.link.see.no_label", "fehlendes Referenzlabel" },
+            { "doclet.link.see.reference_invalid", "Ung\u00FCltige Referenz" },
+            { "doclet.link.see.reference_not_accessible", "Referenz nicht zug\u00E4nglich: {0}" },
+            { "doclet.link.see.reference_not_found", "Referenz nicht gefunden: {0}" },
             { "doclet.module", "Modul" },
             { "doclet.navAnnotationTypeMember", "Element" },
             { "doclet.navAnnotationTypeOptionalMember", "Optional" },
@@ -266,8 +283,6 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.search.main_heading", "Suchen" },
             { "doclet.search.redirect", "Zum ersten Ergebnis umleiten" },
             { "doclet.search.show_more", "Zus\u00E4tzliche Ressourcen" },
-            { "doclet.see.class_or_package_not_accessible", "Tag {0}: Referenz nicht zug\u00E4nglich: {1}" },
-            { "doclet.see.class_or_package_not_found", "Tag {0}: Referenz nicht gefunden: {1}" },
             { "doclet.see.nested_link", "Tag {0}: Verschachtelter Link" },
             { "doclet.systemProperties", "Systemeigenschaften" },
             { "doclet.systemPropertiesSummary", "Systemeigenschaften - \u00DCbersicht" },
@@ -322,6 +337,7 @@ public final class standard_de extends java.util.ListResourceBundle {
             { "doclet.usage.linksource.description", "Quelle in HTML generieren" },
             { "doclet.usage.main-stylesheet.description", "Datei zum \u00C4ndern des Stils der generierten Dokumentation" },
             { "doclet.usage.main-stylesheet.parameters", "<Datei>" },
+            { "doclet.usage.no-fonts.description", "Keine Standardwebschriftarten in generierte Dokumentation aufnehmen" },
             { "doclet.usage.no-frames.description", "Deaktiviert die Verwendung von Frames in der generierten Ausgabe (Standard)" },
             { "doclet.usage.no-module-directories.description", "Gruppiert Dateien f\u00FCr die Moduldokumentation nicht in \nmodulspezifischen Verzeichnisse" },
             { "doclet.usage.no-platform-links.description", "Generiert keine Links zur Plattformdokumentation" },
